@@ -19,14 +19,14 @@ const XML_NAMESPACE = "http://www.w3.org/XML/1998/namespace";
 const XMLNS_NAMESPACE = "http://www.w3.org/2000/xmlns/";
 
 const rootNS: Record<string, string> = {
-  // eslint-disable-next-line @typescript-eslint/no-explicit-any
+  // eslint-disable-next-line @typescript-eslint/no-explicit-any, @typescript-eslint/no-unsafe-assignment
   __proto__: null as any,
   xml: XML_NAMESPACE,
   xmlns: XMLNS_NAMESPACE,
 };
 
 const XML_ENTITIES: Record<string, string> = {
-  // eslint-disable-next-line @typescript-eslint/no-explicit-any
+  // eslint-disable-next-line @typescript-eslint/no-explicit-any, @typescript-eslint/no-unsafe-assignment
   __proto__: null as any,
   amp: "&",
   gt: ">",
@@ -114,7 +114,7 @@ const DTD_TERMINATOR = [...QUOTES, LESS, CLOSE_BRACKET];
 const XML_DECL_NAME_TERMINATOR = [EQUAL, QUESTION, ...S_LIST];
 const ATTRIB_VALUE_UNQUOTED_TERMINATOR = [...S_LIST, GREATER, AMP, LESS];
 
-function nsPairCheck(parser: SaxesParser<{}>, prefix: string,
+function nsPairCheck(parser: SaxesParser<SaxesOptions>, prefix: string,
                      uri: string): void {
   switch (prefix) {
     case "xml":
@@ -153,8 +153,7 @@ ${XMLNS_NAMESPACE}.`);
   }
 }
 
-
-function nsMappingCheck(parser: SaxesParser<{}>,
+function nsMappingCheck(parser: SaxesParser<SaxesOptions>,
                         mapping: Record<string, string>): void {
   for (const local of Object.keys(mapping)) {
     nsPairCheck(parser, local, mapping[local]);
@@ -227,7 +226,6 @@ export type TextHandler = (text: string) => void;
  */
 export type PIHandler = (data: { target: string; body: string }) => void;
 
-
 /**
  * Event handler for doctype.
  *
@@ -568,6 +566,7 @@ export type StartTagForOptions<O extends SaxesOptions> =
     O extends { xmlns?: false | undefined } ? SaxesStartTagPlain :
       SaxesStartTag;
 
+// eslint-disable-next-line @typescript-eslint/ban-types
 export class SaxesParser<O extends SaxesOptions = {}> {
   private readonly fragmentOpt: boolean;
   private readonly xmlnsOpt: boolean;
@@ -699,8 +698,7 @@ export class SaxesParser<O extends SaxesOptions = {}> {
       // eslint-disable-next-line @typescript-eslint/unbound-method
       this.pushAttrib = this.pushAttribNS;
 
-
-      // eslint-disable-next-line @typescript-eslint/no-explicit-any
+      // eslint-disable-next-line @typescript-eslint/no-explicit-any, @typescript-eslint/no-unsafe-assignment
       this.ns = { __proto__: null as any, ...rootNS };
       const additional = this.opt.additionalNamespaces;
       if (additional != null) {
@@ -833,9 +831,8 @@ export class SaxesParser<O extends SaxesOptions = {}> {
     this.line = 1;
     this.column = 0;
 
-    this.ENTITIES = Object.create(XML_ENTITIES);
+    this.ENTITIES = Object.create(XML_ENTITIES) as Record<string, string>;
 
-    // eslint-disable-next-line no-unused-expressions
     this.readyHandler?.();
   }
 
@@ -876,7 +873,7 @@ export class SaxesParser<O extends SaxesOptions = {}> {
    * @param handler The handler to set.
    */
   on<N extends EventName>(name: N, handler: EventNameToHandler<O, N>): void {
-    // eslint-disable-next-line @typescript-eslint/no-explicit-any
+    // eslint-disable-next-line @typescript-eslint/no-explicit-any, @typescript-eslint/no-unsafe-member-access
     (this as any)[EVENT_NAME_TO_HANDLER_NAME[name]] = handler;
   }
 
@@ -886,7 +883,7 @@ export class SaxesParser<O extends SaxesOptions = {}> {
    * @parma name The event to stop listening to.
    */
   off(name: EventName): void {
-    // eslint-disable-next-line @typescript-eslint/no-explicit-any
+    // eslint-disable-next-line @typescript-eslint/no-explicit-any, @typescript-eslint/no-unsafe-member-access
     (this as any)[EVENT_NAME_TO_HANDLER_NAME[name]] = undefined;
   }
 
@@ -942,7 +939,9 @@ export class SaxesParser<O extends SaxesOptions = {}> {
    *
    * @returns this
    */
-  write(chunk: string | {} | null): this {
+  // We do need object for the type here. Yes, it often causes problems
+  // but not in this case.
+  write(chunk: string | object | null): this {
     if (this.closed) {
       return this.fail("cannot write after close; assign an onready handler.");
     }
@@ -970,8 +969,8 @@ export class SaxesParser<O extends SaxesOptions = {}> {
       this.carriedFromPrevious = undefined;
     }
 
-    let limit = (chunk as string).length;
-    const lastCode = (chunk as string).charCodeAt(limit - 1);
+    let limit = chunk.length;
+    const lastCode = chunk.charCodeAt(limit - 1);
     if (!end &&
         // A trailing CR or surrogate must be carried over to the next
         // chunk.
@@ -979,16 +978,16 @@ export class SaxesParser<O extends SaxesOptions = {}> {
       // The chunk ends with a character that must be carried over. We cannot
       // know how to handle it until we get the next chunk or the end of the
       // stream. So save it for later.
-      this.carriedFromPrevious = (chunk as string)[limit - 1];
+      this.carriedFromPrevious = chunk[limit - 1];
       limit--;
-      chunk = (chunk as string).slice(0, limit);
+      chunk = chunk.slice(0, limit);
     }
 
     const { stateTable } = this;
-    this.chunk = chunk as string;
+    this.chunk = chunk;
     this.i = 0;
     while (this.i < limit) {
-      // eslint-disable-next-line @typescript-eslint/no-explicit-any
+      // eslint-disable-next-line @typescript-eslint/no-explicit-any, @typescript-eslint/no-unsafe-argument
       stateTable[this.state].call(this as any);
     }
     this.chunkPosition += limit;
@@ -1088,7 +1087,6 @@ export class SaxesParser<O extends SaxesOptions = {}> {
     return final;
   }
 
-
   /**
    * Get a single code point out of the current chunk. This updates the current
    * position if we do position tracking.
@@ -1362,7 +1360,6 @@ export class SaxesParser<O extends SaxesOptions = {}> {
     const c = this.captureTo(DOCTYPE_TERMINATOR);
     switch (c) {
       case GREATER: {
-        // eslint-disable-next-line no-unused-expressions
         this.doctypeHandler?.(this.text);
         this.text = "";
         this.state = S_TEXT;
@@ -1632,7 +1629,6 @@ export class SaxesParser<O extends SaxesOptions = {}> {
     const c = this.getCodeNorm();
     if (c === MINUS) {
       this.state = S_COMMENT_ENDED;
-      // eslint-disable-next-line no-unused-expressions
       this.commentHandler?.(this.text);
       this.text = "";
     }
@@ -1677,7 +1673,6 @@ export class SaxesParser<O extends SaxesOptions = {}> {
     const c = this.getCodeNorm();
     switch (c) {
       case GREATER: {
-        // eslint-disable-next-line no-unused-expressions
         this.cdataHandler?.(this.text);
         this.text = "";
         this.state = S_TEXT;
@@ -1777,7 +1772,6 @@ export class SaxesParser<O extends SaxesOptions = {}> {
         this.fail(
           "the XML declaration must appear at the start of the document.");
       }
-      // eslint-disable-next-line no-unused-expressions
       this.piHandler?.({
         target: piTarget,
         body: this.text,
@@ -1981,7 +1975,6 @@ export class SaxesParser<O extends SaxesOptions = {}> {
                this.xmlDeclExpects.includes("version")) {
         this.fail("XML declaration must contain a version.");
       }
-      // eslint-disable-next-line no-unused-expressions
       this.xmldeclHandler?.(this.xmlDecl);
       this.name = "";
       this.piTarget = this.text = "";
@@ -2010,10 +2003,9 @@ export class SaxesParser<O extends SaxesOptions = {}> {
     this.name = "";
 
     if (this.xmlnsOpt) {
-      this.topNS = tag.ns = Object.create(null);
+      this.topNS = tag.ns = Object.create(null) as Record<string, string>;
     }
 
-    // eslint-disable-next-line no-unused-expressions
     this.openTagStartHandler?.(tag as StartTagForOptions<O>);
     this.sawRoot = true;
     if (!this.fragmentOpt && this.closedRoot) {
@@ -2405,7 +2397,6 @@ export class SaxesParser<O extends SaxesOptions = {}> {
     const { prefix, local } = this.qname(name);
     const attr = { name, prefix, local, value };
     this.attribList.push(attr);
-    // eslint-disable-next-line no-unused-expressions
     this.attributeHandler?.(attr as AttributeEventForOptions<O>);
     if (prefix === "xmlns") {
       const trimmed = value.trim();
@@ -2425,7 +2416,6 @@ export class SaxesParser<O extends SaxesOptions = {}> {
   private pushAttribPlain(name: string, value: string): void {
     const attr = { name, value };
     this.attribList.push(attr);
-    // eslint-disable-next-line no-unused-expressions
     this.attributeHandler?.(attr as AttributeEventForOptions<O>);
   }
 
@@ -2449,12 +2439,10 @@ export class SaxesParser<O extends SaxesOptions = {}> {
     }
     const { text } = this;
     if (text.length !== 0) {
-      // eslint-disable-next-line no-unused-expressions
       this.textHandler?.(text);
       this.text = "";
     }
     this._closed = true;
-    // eslint-disable-next-line no-unused-expressions
     this.endHandler?.();
     this._init();
     return this;
@@ -2602,7 +2590,6 @@ export class SaxesParser<O extends SaxesOptions = {}> {
 
     // There cannot be any pending text here due to the onopentagstart that was
     // necessarily emitted before we get here. So we do not check text.
-    // eslint-disable-next-line no-unused-expressions
     this.openTagHandler?.(tag as TagForOptions<O>);
     tags.push(tag);
     this.state = S_TEXT;
@@ -2623,9 +2610,7 @@ export class SaxesParser<O extends SaxesOptions = {}> {
 
     // There cannot be any pending text here due to the onopentagstart that was
     // necessarily emitted before we get here. So we do not check text.
-    // eslint-disable-next-line no-unused-expressions
     this.openTagHandler?.(tag as TagForOptions<O>);
-    // eslint-disable-next-line no-unused-expressions
     this.closeTagHandler?.(tag as TagForOptions<O>);
     const top = this.tag = tags[tags.length - 1] ?? null;
     if (top === null) {
@@ -2659,7 +2644,6 @@ export class SaxesParser<O extends SaxesOptions = {}> {
     while (l-- > 0) {
       const tag = this.tag = tags.pop() as SaxesTag;
       this.topNS = tag.ns!;
-      // eslint-disable-next-line no-unused-expressions
       handler?.(tag as TagForOptions<O>);
       if (tag.name === name) {
         break;
@@ -2685,7 +2669,6 @@ export class SaxesParser<O extends SaxesOptions = {}> {
    */
   private parseEntity(entity: string): string {
     // startsWith would be significantly slower for this test.
-    // eslint-disable-next-line @typescript-eslint/prefer-string-starts-ends-with
     if (entity[0] !== "#") {
       const defined = this.ENTITIES[entity];
       if (defined !== undefined) {
